//bunny.cpp

#include "stdio.h"

#include "bunny.h"
#include "blaster.h"
#include "errors.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//bunny implememntation
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


bunny::bunny(int x, int y, int dia)
{
  xpos = x;
  ypos = y;
  diameter = dia;
  pbnext = 0;
  xshift = 0;
  yshift = 0;
}

bunny::~bunny()
{

}

int bunny::getX()
{
  return xpos;
}

int bunny::getY()
{
  return ypos;
}

int bunny::getDia()
{
  return diameter;
}

void bunny::setDia(int dia)
{
	if (dia > maxbunnydia) diameter = maxbunnydia;
	else if (dia < minbunnydia) diameter = minbunnydia;
	else diameter = dia;
}


bunny * bunny::getNext()
{
  return pbnext;
}

void bunny::setNext(bunny * pb)
{
  pbnext = pb;
}

//The shift values are used for bunnies that are close to the edge
//of the image pixmap. Pixels that are off the pixmap have to be diregarded
//when creating "paint", etc.
int bunny::getXShift()
{
  return xshift;
}

int bunny::getYShift()
{
  return yshift;
}

void bunny::setXShift(int shift)
{
  xshift = shift;
}

void bunny::setYShift(int shift)
{
  yshift = shift;
}

bool bunny::isShifted()
{
  if (xshift != 0 || yshift != 0) return true;
  return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//bunnies implementation
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bunnies::bunnies()
{
  pbstart = 0;
  numbunnies = 0;
}

bunnies::~bunnies()
{
  bunny *pb = pbstart;
  bunny *pbnext;
  
  while (pb != 0)
  {
    pbnext = pb->getNext();
    delete pb;
    pb = pbnext;
  }
}
  
int bunnies::add(bunny *pbunny)
{
  bunny * pb;
  bunny * pbnext;
  
  if (pbunny == 0) {
	  ERROR(ERR_NULL_OBJECT, "bunnies::add(), NULL bunny");
	  return 0;
  }
  
  if (numbunnies == 0) 
  {
    pbstart = pbunny;
  }
  else
  {
    pb = pbstart;
    while (pb != 0)
    {
      if ( (pbnext = pb->getNext()) == 0) 
      {
        pb->setNext(pbunny);
      }
      pb = pbnext;
    }  
  }
  
  numbunnies++;
  return numbunnies;
}

int bunnies::remove(bunny *pb)
{
	if (pb == 0) {
		ERROR(ERR_NULL_OBJECT, "bunnies::remove(), NULL bunny");
		return 0;
	}
  
  if (numbunnies == 0) return 0;
  int x = pb->getX();
  int y = pb->getY();
  
  if (x == pbstart->getX() && y == pbstart->getY())
  {
    bunny * pbsave = pbstart;
    pbstart = pb->getNext();
    delete pbsave;
    numbunnies--;
  }
  else
  {
    bunny * p = pbstart->getNext();
    bunny * pbprev = pbstart;
    while (p != 0)
    {
      if (x == p->getX() && y == p->getY())
      {
        pbprev->setNext(p->getNext());
        delete p;
        p = 0;
        numbunnies--;
      }
      else
      {
        pbprev = p;
        p = p->getNext();
      }
    }
  }
  return numbunnies;
}

//Uses 0 based indexing
bunny * bunnies::getAt(int n)
{
  bunny *pb = pbstart;
  int count = 0;
  
  if (n < 0) {
	  ERROR(ERR_BAD_INDEX, "bunnies::getAt(), index < 0");
	  return 0;
  }
  
  while (pb != 0)
  {
    if (count == n) break;
    count++;
    pb = pb->getNext();
  }
  
  return pb;
}

bunny * bunnies::get(int x, int y)
{
	if ( x < 0 || y < 0 ) {
		ERROR(ERR_BAD_COORDS, "bunnies::get(), bunny x or y < 0");
		return 0;
	}
  
  bunny *pb = pbstart;
  while (pb != 0)
  {
    if ( x == pb->getX() && y == pb->getY() ) break;
    pb = pb->getNext();
  }
  
  return pb;
}

bunny * bunnies::contains(int x, int y)
{
	bunny *pb = pbstart;
	while (pb != 0)
	{
		if ( blaster::radius( pb->getX(), pb->getY(), x, y ) <= ( pb->getDia() / 2 ) ) break;	
		pb = pb->getNext();
	}
	
	return pb;
}

int bunnies::save( char* sFilename )
{
	FILE *fp;
	bunny* bp;

	if ( ( bp = getAt(0) ) == 0 )
	{
		return 0;
	}

	if ( (fp = fopen( sFilename, "w" ) ) == NULL )
	{
		ERROR(ERR_NOFILE, "bunnies::save(), fopen failed");
		return ERR_NOFILE;
	}

	do
	{
		fprintf( fp, "%d,%d,%d,%d,%d\n", bp->getX(), bp->getY(), bp->getDia(), bp->getXShift(), bp->getYShift() );
		bp = bp->getNext();
	} while ( bp != ( bunny* ) 0 );

	fclose( fp );	

	return ERR_NOERR;
}

int bunnies::loadFromFile(char* sFilename)
{
	FILE* fp;
	bunny* bp;
	
	if ( ( fp = fopen( sFilename, "r" ) ) == NULL )
	{
		ERROR(ERR_NOFILE, "bunnies::loadFromFile(), fopen failed");
		return ERR_NOFILE;
	}
	
	if ( ( bp = getAt(0) ) != 0 )
	{
		do
		{
			remove(bp);
			bp = getAt(0);
		} while ( bp != ( bunny* ) 0 );
	}
	
	int x, y, dia, xshift, yshift;
	while ( fscanf( fp, "%d,%d,%d,%d,%d\n", &x, &y, &dia, &xshift, &yshift) != EOF)
	{
		bp = new bunny(x, y, dia);
		bp->setXShift(xshift);
		bp->setYShift(yshift);
		add(bp);
	}
	
	return ERR_NOERR;
}








