//canvasview.cpp
#include <qpainter.h>
//Added by qt3to4:
#include <QMouseEvent>
#include <QKeyEvent>
#include <Q3PopupMenu>
#include <stdlib.h>
#include <math.h>
#include <qmenubar.h>

#include "canvasview.h"
#include "bunny.h"
#include "mainwin.h"
#include "blaster.h"
#include "errors.h"
#include "settings.h"

//#define DEBUGOUT

#ifdef DEBUGOUT

#include <iostream>
#define cout std::cout
#define endl std::endl

#endif

extern bunnies * pBunnies;
extern mainwin * pMainWin;

bool bcircledrawn = false;
bunny * pcurrentbunny = ( bunny * ) 0;

canvasview::canvasview(QWidget * parent)
	: Q3CanvasView(parent)
{
	setFocusPolicy( Qt::StrongFocus );
	pmenu = new Q3PopupMenu(this);
	
	pcolormenu = new Q3PopupMenu(this);
	pcolormenu->setTitle("Marker color");
	pcolormenu->insertItem("Red", this, SLOT(slotRedMarker()) );
	pcolormenu->insertItem("Green", this, SLOT(slotGreenMarker()) );
	pcolormenu->insertItem("Blue", this, SLOT(slotBlueMarker()) );
	pcolormenu->insertItem("Yellow", this, SLOT(slotYellowMarker()) );
	
	pmenu->insertItem( "Mark", this, SLOT( slotMark() ) );
	pmenu->insertItem( "Unmark", this, SLOT( slotUnmark() ) );
	pmenu->insertItem( "Move/Resize", this, SLOT( slotMoveResize() ) );
	pmenu->insertSeparator();
	pmenu->addMenu( pcolormenu );
	
	blastMenuId = -1;
	undoMenuId = -1;
}

canvasview::~canvasview()
{

};


void canvasview::mousePressEvent ( QMouseEvent * e )
{
	//for a 3008x2000 image, the MouseEvent contains 1,1 for a click on the 
	//the top left of the canvas and 3009,2001 for a click on the bottom right.

	bcircledrawn = false;

	switch (pMainWin->getState())
	{
		case CREATE_REFERENCE:
		{
			if ( e->button() == Qt::LeftButton ) 
			{
				break;
			}
			else if (e->button() == Qt::RightButton) 
			{
				//display the popup menu at the point the mouse was clicked
				viewportToContents(e->x(), e->y(), centerx, centery);
				
				pmenu->removeItem(blastMenuId);
				pmenu->removeItem(undoMenuId);
				blastMenuId = -1;
				undoMenuId = -1;
				
				QPoint pt( e->x(), e->y() );
				bunny * pb;
				if ( ( pb = pBunnies->contains( centerx, centery ) ) != ( bunny* ) 0 )
				{
					pcurrentbunny = pb;
					pmenu->setItemEnabled( pmenu->idAt(0), false );
					pmenu->setItemEnabled( pmenu->idAt(1), true );
					pmenu->setItemEnabled( pmenu->idAt(2), true );
				}
				else
				{
					pcurrentbunny = ( bunny * ) 0;
					pmenu->setItemEnabled( pmenu->idAt(0), true );
					pmenu->setItemEnabled( pmenu->idAt(1), false );
					pmenu->setItemEnabled( pmenu->idAt(2), false );
				}
				
				pcolormenu->setItemChecked(pcolormenu->idAt(RED),false);
				pcolormenu->setItemChecked(pcolormenu->idAt(GREEN),false);
				pcolormenu->setItemChecked(pcolormenu->idAt(BLUE),false);
				pcolormenu->setItemChecked(pcolormenu->idAt(YELLOW),false);
				pcolormenu->setItemChecked(pcolormenu->idAt(pMainWin->appSettings.markerColor),true);
				
				pmenu->popup( mapToGlobal( pt ) );
			}
			
			break;
		} // end case CREATE_REFERENCE
		
		case OPEN_IMG:
		{
			if ( e->button() == Qt::LeftButton ) 
			{
				break;
			}
			else if (e->button() == Qt::RightButton) 
			{
				//display the popup menu at the point the mouse was clicked
				viewportToContents(e->x(), e->y(), centerx, centery);
				
				if ( blastMenuId == -1 )
				{
					blastMenuId = pmenu->insertItem( "Blast", this, SLOT( slotBlast() ), 0, -1, 3 );
					pmenu->setItemEnabled( pmenu->idAt(3), false );
				}
				if ( undoMenuId == -1 )
				{
					undoMenuId = pmenu->insertItem( "Undo", this, SLOT( slotUndo() ), 0, -1, 4 );
					pmenu->setItemEnabled( pmenu->idAt(4), false );
				}
				
				QPoint pt( e->x(), e->y() );
				bunny * pb;
				if ( ( pb = pBunnies->contains( centerx, centery ) ) != ( bunny* ) 0 )
				{
					pcurrentbunny = pb;
					pmenu->setItemEnabled( pmenu->idAt(0), false );
					pmenu->setItemEnabled( pmenu->idAt(1), true );
					pmenu->setItemEnabled( pmenu->idAt(2), true );
					pmenu->setItemEnabled( pmenu->idAt(3), true );	
				}
				else
				{
					pcurrentbunny = ( bunny * ) 0;
					if (pBunnies->getNumBunnies() > 0)
					{
						pmenu->setItemEnabled( pmenu->idAt(0), false );
						pmenu->setItemEnabled( pmenu->idAt(3), true );	
					}
					else
					{
						pmenu->setItemEnabled( pmenu->idAt(0), true );
						pmenu->setItemEnabled( pmenu->idAt(3), false );	
					}
					pmenu->setItemEnabled( pmenu->idAt(1), false );
					pmenu->setItemEnabled( pmenu->idAt(2), false );
				}
				
				pcolormenu->setItemChecked(pcolormenu->idAt(RED),false);
				pcolormenu->setItemChecked(pcolormenu->idAt(GREEN),false);
				pcolormenu->setItemChecked(pcolormenu->idAt(BLUE),false);
				pcolormenu->setItemChecked(pcolormenu->idAt(YELLOW),false);
				pcolormenu->setItemChecked(pcolormenu->idAt(pMainWin->appSettings.markerColor),true);
				
				if (pMainWin->pixList()->count() > 0) 
				{
					pmenu->setItemEnabled( pmenu->idAt(4), true );
				}
				else
				{
					pmenu->setItemEnabled( pmenu->idAt(4), false );
				}
				
				pmenu->popup( mapToGlobal( pt ) );
			}
			break;
		}// end case OPEN_IMG

		default:
		{
			e->ignore();
			break;
		}  
	} //end switch 

}

//Drag the marker circle with the mouse
void canvasview::mouseMoveEvent ( QMouseEvent * e )
{
	switch (pMainWin->getState())
	{
		case CREATE_REFERENCE:
		case OPEN_IMG:
		{
			viewportToContents(e->x(), e->y(), centerx, centery);
			bunny * pb;
			if ( ( pb = pBunnies->contains( centerx, centery ) ) != ( bunny* ) 0 )
			{
				pb->setX(centerx);
				pb->setY(centery);
				updateCanvas(pb);
				pcurrentbunny = pb;
				bcircledrawn = true;
			}
			default:
			{
				e->ignore();
				break;
			}
		}
	} //end switch
}

//Resize the marker circle with the mousewheel
void canvasview::wheelEvent (QWheelEvent * e)
{
	switch (pMainWin->getState())
	{
		case CREATE_REFERENCE:
		case OPEN_IMG:
		{
			int x, y;
			viewportToContents(e->x(), e->y(), x, y);
			bunny * pb;
			if ( ( pb = pBunnies->contains( x, y ) ) != ( bunny* ) 0 )
			{
				int dia = pb->getDia();
				if (e->delta() > 0) dia += 2;
				else if (e->delta() < 0) dia -= 2;
				if (dia > 2)
				{
					pb->setDia(dia);
					updateCanvas(pb);
					pcurrentbunny = pb;
					bcircledrawn = true;
				}	
			}
			else
			{
				if (e->orientation() == Qt::Vertical)
					scrollBy(0,-(e->delta()));
				else if (e->orientation() == Qt::Horizontal)
					scrollBy(-(e->delta()),0);
			}
			break;
		}
		default:
		{
			if (e->orientation() == Qt::Vertical)
				scrollBy(0,-(e->delta()));
			else if (e->orientation() == Qt::Horizontal)
				scrollBy(-(e->delta()),0);
			break;
		}
	}
}


//Handle arrow keys and plus/minus keys to adjust
//the size and position of the bunny circle.
void canvasview::keyPressEvent ( QKeyEvent *e )
{
	if ( ( pMainWin->getState()  == CREATE_REFERENCE || pMainWin->getState() == OPEN_IMG ) 
		  && bcircledrawn == true )
	{
		bunny * pb;
		if ( pcurrentbunny )
		{
			pb = pcurrentbunny;
			pcurrentbunny = (bunny *) 0;
		}
		else
		{
			pb = pBunnies->getAt( pBunnies->getNumBunnies( ) - 1 );
		}
		if ( pb == ( bunny* ) 0 ) { 
			ERROR(ERR_NULL_OBJECT, "canvasview::keyPressEvent(), pb == NULL");
			return;
		}
		
		//adust the bunny size or position based on keypress
		centerx = pb->getX( );
		centery = pb->getY( );
		int dia = pb->getDia( );

		if ( e->key( ) == Qt::Key_Right ) centerx++;
		else if ( e->key( ) == Qt::Key_Left ) centerx--;
		else if ( e->key( ) == Qt::Key_Up ) centery--;
		else if ( e->key( ) == Qt::Key_Down ) centery++;
		else if ( e->key( ) == Qt::Key_Plus ) dia += 2; 
		else if ( e->key( ) == Qt::Key_Minus ) dia -= 2;
        else 
		{
			e->ignore( );
			return;
		}
		
		//update the bunny list with the new bunny specs
		pb->setX(centerx);
		pb->setY(centery);
		pb->setDia(dia);
		pcurrentbunny = pb;

		updateCanvas(pb);
	}
}

void canvasview::slotMark()
{
	//centerx and centery were initialized in the mousePressEvent()
	//handler when the user right clicked on the bunny
	int dia = 18;

	//allocate a bunny and add it to the bunnies list.
	bunny *pb = new bunny(centerx, centery, dia);
	pBunnies->add(pb);

	//force the canvas to update
	//without this the circle around the bunnies doesnt show up
	updateCanvas(pb);
	bcircledrawn = true;
	pMainWin->pstatuslabel->setText("Use arrow and plus/minus keys to move/resize circle.");
}


void canvasview::slotMoveResize()
{
	bcircledrawn = true;
}

void canvasview::slotUnmark()
{
	bunny * pb;
	if ( ( pb = pBunnies->contains(centerx, centery)) != ( bunny* ) 0 )
	{
		pBunnies->remove( pb );
		updateCanvas((bunny*)0);
	}
	
	if ( pBunnies->getNumBunnies() == 0 ) 
	{
		bcircledrawn = false;
	}
	else
	{	
		bcircledrawn = true;
	}
}


void canvasview::slotRedMarker()
{
	pMainWin->appSettings.markerColor = RED;	
	canvas()->setAllChanged();
	canvas()->update();
}

void canvasview::slotGreenMarker()
{
	pMainWin->appSettings.markerColor = GREEN;	
	canvas()->setAllChanged();
	canvas()->update();
}

void canvasview::slotBlueMarker()
{
	pMainWin->appSettings.markerColor = BLUE;	
	canvas()->setAllChanged();
	canvas()->update();
}

void canvasview::slotYellowMarker()
{
	pMainWin->appSettings.markerColor = YELLOW;	
	canvas()->setAllChanged();
	canvas()->update();
}

void canvasview::slotBlast()
{
	if (pBunnies == 0) return;
	if (pMainWin->pixmap() == 0) return;
	
	blaster *pblast = new blaster();
	
	//Add the bunny rect pixmap to the undo buffer
	QPixmap pm;
	QPoint pt;
	pm = pblast->getBunnyPixmap(pBunnies, 0, pMainWin->pixmap(), pt);
	if ( pm.isNull() == false ) 
	{
		pMainWin->pixList()->add(pm, pt); 
	}
	
	pblast->interpolate(pBunnies, 0, pMainWin->pixmap());
	
	canvas()->setAllChanged();
	canvas()->update();
	
	delete pblast;
	pblast = 0;
	
	pMainWin->setState(OPEN_IMG, OPEN_IMG_DIRTY);
}

void canvasview::slotUndo()
{
	if (pBunnies == 0) return;
	if (pMainWin->pixmap() == 0) return;
	
	blaster *pblast = new blaster();
	
	int index = pMainWin->pixList()->count() -1;
	if (index < 0) return;
	
	QPoint pt;
	QPixmap pmp = pMainWin->pixList()->get(index, pt);
	
	if (pblast->undoBlast(pMainWin->pixmap(), pmp, pt) == ERR_NOERR)
	{
		pMainWin->pixList()->remove(index); 
	}
		
	canvas()->setAllChanged();
	canvas()->update();
	
	delete pblast;
	pblast = 0;
	
	pMainWin->setState(OPEN_IMG, OPEN_IMG_DIRTY);
}


void canvasview::updateCanvas(bunny *pb)
{
	int x,y,dia;
	 
	if (pb != (bunny *) 0)
	{
		x = pb->getX();
		y = pb->getY();
		dia = pb->getDia();
	}
	
	if (pb == (bunny *) 0)
	{
		canvas()->setAllChanged();
	}
	else
	{
		int pad = 32;
		QPoint topLeft(x - ((dia/2)+pad), y - ((dia/2)+pad) );
		QPoint bottomRight( x + ((dia/2)+pad), y + ((dia/2)+pad) );
		QRect bunnyRect(topLeft, bottomRight);
		canvas()->setChanged(bunnyRect);
	}
	canvas()->update();
	
	if (pMainWin->getState() == CREATE_REFERENCE) 
	{
		pMainWin->setState( CREATE_REFERENCE, CREATE_REFERENCE_DIRTY);
	}
}
