//
// C++ Implementation: cmdline
//
// Description: 
//
//
// Author: ,,, <heinzg@denizen>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <cstdlib>
#include <qapplication.h>
#include <qstringlist.h>
#include <qdir.h>
#include <qimage.h>

#include "cmdline.h"
#include "errors.h"
#include "bunny.h"
#include "blaster.h"
#include "mainwin.h"
#include "logger.h"
#include "version.h"
#include "exif.h"
#include "hotpixel.h"


//#define DEBUGOUT

#ifdef DEBUGOUT
#include <iostream>
#define cout std::cout
#define endl std::endl
#endif //DEBUGOUT

extern mainwin* pMainWin;

QStringList gslArgsList;
settings tempsettings;

int readCmdLineArgs()
{
	for (int i = 0; i < qApp->argc(); i++)
	{
		gslArgsList += qApp->argv()[i];
	}
	
	if ( gslArgsList.find("--batch") != gslArgsList.end() )
	{
		return -1;
	}
	
	if ( gslArgsList.find("-h") != gslArgsList.end() || 
		 gslArgsList.find("--help") != gslArgsList.end() ||
		 gslArgsList.find("-?") != gslArgsList.end()
	   )
	{
		return -1;
	}
	
	if (qApp->argc() != 1)
	{
		return -1;
	}
	
	return ERR_NOERR;
}

int processCmdLineArgs()
{
	QStringList::iterator it;
	
	if ( (it = gslArgsList.find("-v") ) !=  gslArgsList.end() )
	{
		fprintf(stdout, "%s Version: %s\n", qApp->arguments().at(0).latin1(), DBBLASTVERSION );
		return ERR_NOERR;
	}
	
	if ( gslArgsList.find("-h") != gslArgsList.end() || 
			gslArgsList.find("--help") != gslArgsList.end() ||
			gslArgsList.find("-?") != gslArgsList.end()
	   )
	{
		displayUsage();
		return ERR_NOERR;
	}
	
	//Copy appSettinngs in case we override the settings
	tempsettings = pMainWin->appSettings;
	
	//process -t (threshhold disabled)
	if ( (it = gslArgsList.find("-t") ) !=  gslArgsList.end() )
	{
		pMainWin->appSettings.bThreshholdsDisabled = true;
	}
	else
	{
		//threshholds will be enabled if -l or -r is used
		//process -l (lightness) 
		if ( (it = gslArgsList.find("-l") ) !=  gslArgsList.end() )
		{
			if (++it != gslArgsList.end())
			{
				bool ok;
				int lightness = (*it).toInt(&ok);
				if (ok && ( lightness >= 0 && lightness <= 255) )
				{
					pMainWin->appSettings.avgColorLightness = lightness;
					pMainWin->appSettings.bThreshholdsDisabled = false;
				}
				else
				{
					//print error message, restore appSettings, exit
					fprintf(stdout, "-l: requires lightness value between 0 and 255\n");
					pMainWin->appSettings = tempsettings;
					exit(1);
				}
			}
		}
		
		//process -r (range)
		if ( (it = gslArgsList.find("-r") ) !=  gslArgsList.end() )
		{
			if (++it != gslArgsList.end())
			{
				bool ok;
				int range = (*it).toInt(&ok);
				if (ok && ( range >= 0 && range <= 255) )
				{
					pMainWin->appSettings.avgColorRange = range;
					pMainWin->appSettings.bThreshholdsDisabled = false;
				}
				else
				{
					//print error message, restore appSettings, exit
					fprintf(stdout, "-r: requires range value between 0 and 255\n");
					pMainWin->appSettings = tempsettings;
					exit(1);
				}
			}
		}
	}
	
	//process -s (save quality)
	if ( (it = gslArgsList.find("-s") ) !=  gslArgsList.end() )
	{
		if (++it != gslArgsList.end())
		{
			bool ok;
			int saveq = (*it).toInt(&ok);
			if (ok && ( saveq >= 30 && saveq <= 100) )
			{
				pMainWin->appSettings.saveImgQuality = saveq;
			}
			else
			{
				//print error message, restore appSettings, exit
				fprintf(stdout, "-s: requires save quality value between 30 and 100\n");
				pMainWin->appSettings = tempsettings;
				exit(1);
			}
		}
	}
	
	if ( (it = gslArgsList.find("-d") ) !=  gslArgsList.end() )
	{
		if (++it != gslArgsList.end())
		{
			//(*it).latin1() will be the name of the .dbr file
			//batch process files in the current directory
			QFileInfo f((*it).latin1());
			if (f.exists())
			{
				int ret = batchProcess(*it);
				pMainWin->appSettings = tempsettings;
				return ret;
			}
			else
			{
				pMainWin->appSettings = tempsettings;
				fprintf(stdout, "File does not exist: %s\n", (*it).latin1());
				exit(1);
			}
		}
	}
	
	if ( (it = gslArgsList.find("-o") ) !=  gslArgsList.end() )
	{
		if (++it != gslArgsList.end())
		{
			//(*it).latin1() will be the name of the output directory
			pMainWin->appSettings.sOutputDir = (*it).latin1();
		}
	}
	
	if ( (it = gslArgsList.find("-n") ) !=  gslArgsList.end() )
	{
		pMainWin->appSettings.bHotpixelsDisabled = true;
	}
	
	if ( (it = gslArgsList.find("--batch") ) !=  gslArgsList.end() && 
		 (it = gslArgsList.find("-p") ) !=  gslArgsList.end() &&
		 pMainWin->appSettings.bHotpixelsDisabled == false )
	{
		//process hotpixels only
		int ret = processHotPixels();
		pMainWin->appSettings = tempsettings;
		return ret;
		
	}
	
	else if ( (it = gslArgsList.find("--batch") ) !=  gslArgsList.end() )
	{
		//batch process images in the current directory and 
		//using default.dbr in the current directory
		int ret = batchProcess("default.dbr");
		pMainWin->appSettings = tempsettings;
		return ret;
	}
	
	pMainWin->appSettings = tempsettings;
	displayUsage();
	return ERR_NOERR;			
}

void displayUsage()
{
	fprintf(stdout, "Usage: %s --batch\n", qApp->arguments().at(0).latin1());
	fprintf(stdout, "Optional args: -d dbrfile -v -h\n");
	fprintf(stdout, "[-t || -l nnn -r nnn] -s nnn -o outputdir -p -n\n");
#ifndef WINDOWS_CONSOLE	
	fprintf(stdout, "Used without arguments, starts the dbblast GUI program.\n");
#endif
	fprintf(stdout, "--batch: Runs in batch mode from the command line;\n");
	fprintf(stdout, "  processes all .jpg files in the current directory,\n");
	fprintf(stdout, "  using the default.dbr file in the current directory.\n");
	fprintf(stdout, "-d dbrfile: Uses the specified .dbr file.\n");
	fprintf(stdout, "-o outputdir: Uses the specified output directory.\n");
	fprintf(stdout, "  outputdir can be either relative or absolute path.\n");
	fprintf(stdout, "-t: disables threshhold checking \n");
	fprintf(stdout, "-l nnn: Sets the average lightness threshhold to nnn.\n");
	fprintf(stdout, "-r nnn: Sets the color range threshhold to nnn.\n");
	fprintf(stdout, "  For either -l or -r use a value between 0 and 255.\n");
	fprintf(stdout, "  Both -l and -r are ignored if -t is specified.\n");
	fprintf(stdout, "-s nnn: Sets the jpeg save quality to nnn.\n");
	fprintf(stdout, "  For -s use a value between 30 and 100.\n");
	fprintf(stdout, "-p: Batch process hotpixels only.\n");
	fprintf(stdout, "  Requires the file default default.hpx containing list of\n");
	fprintf(stdout, "  hot pixels in the current directory.\n");
	fprintf(stdout, "-n: disable hotpixel processing.\n");
	fprintf(stdout, "-v: Displays version number and exits.\n");
	fprintf(stdout, "-h: Displays this usage message.\n\n");
}

int batchProcess(QString sDbrFile)
{
	int ret;
	QStringList slImgList; 
	QStringList sListHotPixels;
	bunnies* pBunnies;
	FILE * fpLog;
	pBunnies = new bunnies();
	
	if (pBunnies == (bunnies*)0)
	{
		ERROR(ERR_NULL_OBJECT, "cmdline.cpp, batchProcess(), pBunnies == 0");
		return ERR_NULL_OBJECT;
	}

	//init the bunny list and load it from the file
	if ((ret = initBunnies(pBunnies, sDbrFile)) != ERR_NOERR)
	{
		return ret;
	}
	
	//create the list of image files
	if ((ret = loadImgList(slImgList)) != ERR_NOERR )
	{
		return ret;
	}
	
	//create the output directory
	if ((ret = createOutputDir()) != ERR_NOERR )
	{
		return ret;
	}
	
	//Open the log file in the current directory
	if ( (fpLog = fopen("dbblast.log", "w")) == NULL )
	{
		ERROR(ERR_NOFILE, "cmdline.cpp, batchProcess(), fpLog == 0");
		return ERR_NOFILE;
	}
	
	QString sListPath = QDir::currentPath() + "/default.hpx";
	hotpixel::getHotPixelList(sListPath, sListHotPixels);
	
	QImage* pImg;
	QStringList::iterator it;
	int i, n;
	QString sOutPath;
	QString s;
	for (it = slImgList.begin(); it != slImgList.end(); ++it)
	{
		//create the blaster and blast each file
		blaster *pblast = new blaster();
		pblast->fpLogFile = fpLog;
		
		pImg = new QImage(*it);
		
		if (pImg->isNull() == true)
		{
			ERROR(ERR_NULL_OBJECT, "cmdline.cpp, batchProcess(), pImg == 0");
			return ERR_NULL_OBJECT;
		}
		
		s = "FILE ";
		s += (*it);
		LOGGER((char*)s.latin1(), fpLog);
		
		fprintf(stdout, "Blasting %s...\n", (*it).latin1());
		fflush(stdout);
		
		n = pBunnies->getNumBunnies();
		for (i = 0; i < n; i++)
		{
			s.sprintf("  BUNNY %d %d",
					  pBunnies->getAt(i)->getX(), 
					  pBunnies->getAt(i)->getY() );
			LOGGER((char*)s.latin1(), fpLog);
			
			pblast->blastBunnyFromQImage( *pImg, pBunnies, i );
		}
		
		//sOutPath = "dbblast/";
		sOutPath = pMainWin->appSettings.sOutputDir;
		sOutPath += "/";
		sOutPath += *it;
		QFileInfo outInfo(sOutPath);
		sOutPath = outInfo.absFilePath();
		
		if (!sListHotPixels.isEmpty() && pMainWin->appSettings.bHotpixelsDisabled == false)
		{
			hotpixel hp(sListHotPixels);
			
			fprintf(stdout, "Processing hotpixels in %s...\n", (*it).latin1());
			fflush(stdout);
			
			hp.processHotPixels(*pImg);
			
			s = "Processed hotpixels.";
			LOGGER((char*)s.latin1(), fpLog);
		}
		
		s = "ENDFILE ";
		s += (*it);
		LOGGER((char*)s.latin1(), fpLog);
		
		pImg->save( sOutPath, 0, pMainWin->appSettings.saveImgQuality );
		copyExifData( (*it).latin1(), sOutPath.latin1() );
		
		delete pImg;
		pImg = 0;
		
		delete pblast;
		pblast = 0;
	}
	
	fclose(fpLog);
	fprintf(stdout, "Done!\n");
	fflush(stdout);
	
	delete pBunnies;
	pBunnies = 0;
	
	return ERR_NOERR;
}

int processHotPixels()
{
	int ret;
	QStringList slImgList; 
	QStringList sListHotPixels;
	FILE * fpLog;
	
	if (pMainWin->appSettings.bHotpixelsDisabled == true) return ERR_NOERR;
	
	//create the list of image files
	if ((ret = loadImgList(slImgList)) != ERR_NOERR )
	{
		return ret;
	}
	
	//create the output directory
	if ((ret = createOutputDir()) != ERR_NOERR )
	{
		return ret;
	}
	
	//Open the log file in the current directory
	if ( (fpLog = fopen("dbblast.log", "w")) == NULL )
	{
		ERROR(ERR_NOFILE, "cmdline.cpp, batchProcess(), fpLog == 0");
		return ERR_NOFILE;
	}
	
	QString sListPath = QDir::currentPath() + "/default.hpx";
	if ( hotpixel::getHotPixelList(sListPath, sListHotPixels) == ERR_NOFILE )
	{
		ERROR(ERR_NOFILE, "cmdline.cpp, processHotPixels(), default.hpx not found..");
		return ERR_NOFILE;
	}
	
	if ( sListHotPixels.isEmpty() )
	{
		ERROR(ERR_NOFILE, "cmdline.cpp, processHotPixels(), hot pixels list is empty.");
		return ERR_NOFILE;
	}
	
	QImage* pImg;
	QStringList::iterator it;
	QString sOutPath;
	QString s;
	for (it = slImgList.begin(); it != slImgList.end(); ++it)
	{
		pImg = new QImage(*it);
		
		if (pImg->isNull() == true)
		{
			ERROR(ERR_NULL_OBJECT, "cmdline.cpp, batchProcess(), pImg == 0");
			return ERR_NULL_OBJECT;
		}
		
		s = "FILE ";
		s += (*it);
		LOGGER((char*)s.latin1(), fpLog);
		
		fprintf(stdout, "Processing hotpixels in %s...\n", (*it).latin1());
		fflush(stdout);
		
		//sOutPath = "dbblast/";
		sOutPath = pMainWin->appSettings.sOutputDir;
		sOutPath += "/";
		sOutPath += *it;
		QFileInfo outInfo(sOutPath);
		sOutPath = outInfo.absFilePath();
		
		hotpixel hp(sListHotPixels);
		hp.processHotPixels(*pImg);
			
		s = "Processed hotpixels.";
		LOGGER((char*)s.latin1(), fpLog);
		
		s = "ENDFILE ";
		s += (*it);
		LOGGER((char*)s.latin1(), fpLog);
		fflush(fpLog);
		
		pImg->save( sOutPath, 0, pMainWin->appSettings.saveImgQuality );
		copyExifData( (*it).latin1(), sOutPath.latin1() );
		
		delete pImg;
		pImg = 0;
	}
	
	fclose(fpLog);
	fprintf(stdout, "Done!\n");
	fflush(stdout);
	
	return ERR_NOERR;
}

int initBunnies(bunnies* pBunnies, QString sDbrFile)
{
	if ( pBunnies == (bunnies*)0 )
	{
		return ERR_NULL_OBJECT;
	}
	
	return pBunnies->loadFromFile((char*)sDbrFile.latin1());
}
	
int loadImgList(QStringList& slImgList)
{
	QDir currentDir;
	currentDir.setFilter(QDir::Files);
	currentDir.setNameFilter("*.jpg;*.jpeg;*.png");
	slImgList = currentDir.entryList();
	
	if (slImgList.isEmpty() == true)
	{
		ERROR(ERR_NOFILE, "cmdline.cpp, loadImgList()");
		return ERR_NOFILE;
	}
	
	return ERR_NOERR;
}

int createOutputDir()
{
	QDir currentDir;
	if (currentDir.exists(pMainWin->appSettings.sOutputDir))
	{
		return ERR_NOERR;
	}
	
	if (currentDir.mkdir(pMainWin->appSettings.sOutputDir) == false)
	{
		ERROR(ERR_DIR_CREATE_FAILED, "cmdline.cpp, createOutputDir()");
		return ERR_DIR_CREATE_FAILED;
	}
	return ERR_NOERR;
}

