//
// C++ Implementation: dbbatch
//
// Description: 
//
//
// Author: ,,, <heinzg@denizen>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <q3filedialog.h>
//#include <Q3TextStream>
#include <qmessagebox.h>
#include <qimage.h>
#include <qapplication.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qpushbutton.h>
#include <q3progressbar.h>
#include <QFileDialog>

#include "dbbatch.h"
#include "bunny.h"
#include "dbbconfig.h"
#include "blaster.h"
#include "errors.h"
#include "mainwin.h"
#include "logger.h"
#include "errors.h"
#include "exif.h"
#include "hotpixel.h"

#include <QTextStream>


//#define DEBUGOUT

#ifdef DEBUGOUT

#include <iostream>
#define cout std::cout
#define endl std::endl

#endif //DEBUGOUT

extern bunnies * pBunnies;
extern mainwin * pMainWin;


dbbatch::dbbatch( QWidget* parent, const char* name, bool modal, Qt::WFlags fl )
	: QDialog( parent, name, modal, fl) 
{
	setupUi(this);
	setModal(true);
	setSizeGripEnabled(true);
	
	if (pMainWin->appSettings.bHotpixelsOnly)
		btnBlast->setText("&Blast hotpixels");
	
	connect(textLogger, SIGNAL(textChanged()), this, SLOT(slot_textChanged()));
	
	initProgressBar();
	
	state = IDLE;
	enableButtons(state);
};

dbbatch::~dbbatch()
{
};

///////////////////////////////////////////////////////////////////////////
//slots 

void dbbatch::btnLoadFiles_clicked()
{
	Q3FileDialog* fd = new Q3FileDialog( this, "", TRUE );
	
	QString sDir;
	if (pMainWin->appSettings.sLastDir == "") sDir = QDir::home().absPath();
	else sDir = pMainWin->appSettings.sLastDir;
	
	fd->setDir(sDir);
	fd->setMode( Q3FileDialog::ExistingFiles );
	fd->setFilter( "Images (*.jpg;*.jpeg;*.png)" );
	fd->setWindowTitle("Open image files");
	if ( fd->exec() == QDialog::Accepted )
	{
		slImgFileList = fd->selectedFiles();
	}
	if (!slImgFileList.isEmpty())
	{
		if ( pBunnies != 0 && pBunnies->getNumBunnies() > 0 ) state = REFLOADED;
		else state = IMGSLOADED;
		enableButtons(state);
		initProgressBar();
		pMainWin->appSettings.sLastDir = fd->dirPath();
		
		QString sListPath = fd->dirPath() + "/default.hpx";
		hotpixel::getHotPixelList(sListPath, sListHotPixels);
	}
};

void dbbatch::btnLoadRef_clicked()
{
	QString sDir;
	if (pMainWin->appSettings.sLastDir == "") sDir = QDir::home().absPath();
	else sDir = pMainWin->appSettings.sLastDir;
	
	QString s;
	Q3FileDialog* fd = new Q3FileDialog( this, "", TRUE );
	fd->setDir(sDir);
	fd->setMode( Q3FileDialog::ExistingFile );
	fd->setFilter( "Reference files (*.dbr)" );
	fd->setWindowTitle("Open reference file");
	if ( fd->exec() == QDialog::Accepted )
	{
		s = fd->selectedFile();
	}
	
	if ( !s.isEmpty() )
	{
		if (pBunnies != 0) {
			delete pBunnies;
			pBunnies = 0;
		}
		pBunnies = new bunnies(); 
		pBunnies->loadFromFile( (char*) s.latin1() );
		state = REFLOADED;
		enableButtons(state);
		QFileInfo f(s);
		pMainWin->appSettings.sLastDir = f.dirPath(true);;
	}
};

void dbbatch::btnConfig_clicked()
{
	dbbconfig * pconfig = new dbbconfig(this);
	pconfig->show();
};

void dbbatch::btnBlast_clicked()
{
	bCancelBlast = false;
	if (state == BLASTING) {
		bCancelBlast = true;
		return;
	}
	
	initProgressBar();
	
	textLogger->setText("");
	
	//Check that we have bunnies loaded and files loaded
	if (slImgFileList.empty())
	{
		QMessageBox::warning(this, "dbblast", "No images loaded.");
		return;
	}
	
	if (pMainWin->appSettings.bHotpixelsOnly == false)
	{
		if (pBunnies == 0 || pBunnies->getNumBunnies() == 0 )
		{
			QMessageBox::warning(this, "dbblast", "No reference file loaded.");
			return;
		}
	}
	QStringList::iterator it;
	it = slImgFileList .begin();
	
	if (createOutputDir( *it ) != ERR_NOERR)
	{
		return;
	}
	
	int savedstate = state;
	state = BLASTING;
	QString sSavedText = btnBlast->text();
	btnBlast->setText("Cancel");
	enableButtons(BLASTING);

	int count = 0;
	int total = slImgFileList.size();
	progressBarFiles->setTotalSteps(total);
	QString s;
	for ( it = slImgFileList .begin(); it != slImgFileList.end(); ++it )
	{
		if (bCancelBlast == true) {
			break;
		}
		qApp->processEvents();
		
		
		//blast the bunnies in each file
		blastBunnies(*it);
		
		//update the progress bar
		count++;
		progressBarFiles->setProgress(count);
		s.sprintf("%d of %d images processed.", count, total);
		textLabelFiles->setText(s);
	}
	
	if (bCancelBlast == true)
	{
		textLabelFileName->setText("Cancelled!");
	}
	else 
	{
		textLabelFileName->setText("Done!");
	}
	enableButtons(savedstate);
	btnBlast->setText(sSavedText);
	state = savedstate;
};

void dbbatch::btnSaveLog_clicked()
{
	if ( textLogger->length() == 0 ) return;
	
	QString sDir;
	if (pMainWin->appSettings.sLastDir == "") sDir = QDir::home().absPath();
	else sDir = pMainWin->appSettings.sLastDir;
	
	QString s = Q3FileDialog::getSaveFileName(
	sDir,
	"Log files (*.log);;All files (*.*)",
	this,
	"save file dialog",
	"Choose a filename to save under" );
	
	if ( !s.isEmpty() )
	{
		QFileInfo fi(s);
		if ( !fi.suffix().contains("log", false) )
		{
			s += ".log";
		}

		QFile file(s);
		//QIODevice::Text ensures that line endings are \n\r on Win32
		if ( file.open( QIODevice::WriteOnly | QIODevice::Text ) )
		{
			QTextStream stream( &file );
			stream << textLogger->text();
			file.close();
			
			QFileInfo f(s);
			pMainWin->appSettings.sLastDir = f.dirPath(true);;
		}
	}
};

void dbbatch::btnClearLog_clicked()
{
	initProgressBar();
	
	textLogger->setText("");
};

void dbbatch::slot_textChanged()
{
	if (textLogger->length() == 0)
	{
		btnClearLog->setEnabled(false);
		btnSaveLog->setEnabled(false);
	}
}

void dbbatch::btnClose_clicked()
{
	close();
}


//End slots
///////////////////////////////////////////////////////////////////////////


int dbbatch::blastBunnies(QString sFileName)
{
	if (pBunnies == 0 && pMainWin->appSettings.bHotpixelsOnly == false)
	{
		return ERR_NULL_OBJECT;
	}
	
	blaster *pblast = new blaster();
	pblast->pLogEdit = textLogger;
	
	//split the absolute dir path and filename
	QFileInfo info(sFileName);
	QString sDir = info.dirPath(true);
	QString sFile = info.fileName();
	
	//create the output file pathname 
	QString sOutDir;
	QString sOutFileName;
	if (QDir::isRelativePath(pMainWin->appSettings.sOutputDir))
	{
		sOutDir = "/" + pMainWin->appSettings.sOutputDir + "/";
		sOutFileName = sDir + sOutDir + sFile;
	}
	else
	{
		sOutDir = pMainWin->appSettings.sOutputDir + "/";
		sOutFileName = sOutDir + sFile;
	}
	QFileInfo outInfo(sOutFileName);
	sOutFileName = outInfo.absFilePath();
	
	QImage img(sFileName);
	
	if (!img.isNull())
	{
		textLabelFileName->setText(sFile);
		QString s("FILE " + sFileName);
		LOGGER ((char*)s.latin1(), textLogger);
		
		if (pMainWin->appSettings.bHotpixelsOnly == false)
		{
			int n = pBunnies->getNumBunnies();	
			for (int i = 0; i < n; i++)
			{
				s.sprintf("  BUNNY %d %d",
						pBunnies->getAt(i)->getX(), 
						pBunnies->getAt(i)->getY() );
				LOGGER((char*)s.latin1(), textLogger);
				
				pblast->blastBunnyFromQImage( img, pBunnies, i );
			}
		}
		if (!sListHotPixels.isEmpty() && pMainWin->appSettings.bHotpixelsDisabled == false)
		{
			hotpixel hp(sListHotPixels);
			hp.processHotPixels(img);
			
			s = "Processed hotpixels.";
			LOGGER((char*)s.latin1(), textLogger);
		}
		
		s = "ENDFILE ";
		s += sFileName;
		LOGGER((char*)s.latin1(), textLogger);
		
		img.save( sOutFileName, 0, pMainWin->appSettings.saveImgQuality );
		copyExifData(sFileName.latin1() , sOutFileName.latin1() );
	}
	
	delete pblast;
	pblast = 0;
	
	return ERR_NOERR;
}


int dbbatch::createOutputDir(QString sFileName)
{
	QFileInfo info(sFileName);
	QString sDir = info.dirPath(true);
	QString sOutDir(pMainWin->appSettings.sOutputDir);
	
	QDir d(sOutDir);
	d.cd(sDir);
	
	if (!d.exists(sOutDir))
	{
		if ( d.mkdir(sOutDir, false) == false )
		{
			d.cd(qApp->applicationFilePath());
			ERROR( ERR_DIR_CREATE_FAILED, "dbbatch::createOutputDir()");
			return ERR_DIR_CREATE_FAILED;
		}
	}
	d.cd(qApp->applicationFilePath());
	return ERR_NOERR;
}

void dbbatch::enableButtons(int state)
{
	switch (state)
	{
		case IDLE:
			btnLoadRef->setEnabled(false);
			btnBlast->setEnabled(false);
			btnSaveLog->setEnabled(false);
			btnClearLog->setEnabled(false);
			break;
		case IMGSLOADED:
			if (pMainWin->appSettings.bHotpixelsOnly == false) 
			{
				btnLoadRef->setEnabled(true);
				break;
			}
			//else fall though
		case REFLOADED:
			btnLoadFiles->setEnabled(true);
			if (pMainWin->appSettings.bHotpixelsOnly == false) 
			{
				btnLoadRef->setEnabled(true);
			}
			btnConfig->setEnabled(true);
			if (textLogger->length() > 0)
			{
				btnSaveLog->setEnabled(true);
				btnClearLog->setEnabled(true);
			}
			btnClose->setEnabled(true);
			btnBlast->setEnabled(true);
			break;
		case BLASTING:
			btnLoadFiles->setEnabled(false);
			btnLoadRef->setEnabled(false);
			btnConfig->setEnabled(false);
			btnSaveLog->setEnabled(false);
			btnClearLog->setEnabled(false);
			btnClose->setEnabled(false);
			break;
		default:
			break;
	}
}

void dbbatch::initProgressBar()
{
	progressBarFiles->setProgress(0);
	textLabelFileName->setText("No files processed.");
	textLabelFiles->setText("0 of 0 files processed.");
}

void dbbatch::hotPixelProcessingChanged(bool bHotPixelOnly)
{
	if (bHotPixelOnly) 
	{
		btnLoadRef->setEnabled(false);
		btnBlast->setText("&Blast hotpixels");
	}
	if ( (state == IMGSLOADED || state == REFLOADED) &&  bHotPixelOnly == false)
	{
		btnLoadRef->setEnabled(true);
		btnBlast->setText("&Blast");
	}
	if (bHotPixelOnly == true && state == IMGSLOADED)
	{
		btnBlast->setEnabled(true);
	}
	if (bHotPixelOnly == false)
	{
		btnBlast->setText("&Blast");
	}
}




