//
// C++ Implementation: dbbconfig
//
// Description: 
//
//
// Author: ,,, <heinzg@denizen>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <iostream>
#include <qslider.h>
#include <qstring.h>
#include <qlabel.h>
#include <q3filedialog.h>
#include <qcheckbox.h>

#include "dbbconfig.h"
#include "settings.h"
#include "mainwin.h"
#include "dbbatch.h"

//#define DEBUGOUT
#ifdef DEBUGOUT
#include <iostream>
#define cout std::cout
#define endl std::endl
#endif //DEBUGOUT

//Put all custom code for the dbbconfig dialog in this file.
//The dbbconfig.cpp file will get overwritten each time the
//ui compiler runs.

extern mainwin * pMainWin;

dbbconfig::dbbconfig( QWidget* parent, const char* name, bool modal, Qt::WFlags fl ) 
	: QDialog( parent, name, modal, fl) 
{
	setupUi(this);
	
	connect(BtnOutputDir, SIGNAL(clicked()), this, SLOT(BtnOutputDir_clicked()));
	connect(ckboxDisableHotpixels, SIGNAL(clicked()), this, SLOT(ckboxDisableHotpixels_clicked()));
	connect(ckboxHotPixelOnly, SIGNAL(clicked()), this, SLOT(ckboxHotPixelOnly_clicked()));
	connect(btnReset, SIGNAL(clicked()), this, SLOT(btnReset_clicked()));
	connect(btnDefault, SIGNAL(clicked()), this, SLOT(btnDefault_clicked()));
	connect(btnApply, SIGNAL(clicked()), this, SLOT(btnApply_clicked()));
	connect(lineEditOutputDir, SIGNAL(textChanged(const QString&)), this, SLOT(outDirChanged(const QString&)));
	
	setModal(false);
	
	initWidgets();
	
	savedSettings = pMainWin->appSettings;
	
	setState(dbbconfig::NEW);
};

dbbconfig::~dbbconfig()
{
};

void dbbconfig::AvgLightness_sliderMoved( int lightness )
{
	updateAvgLightnessText(lightness);
	setState(dbbconfig::DIRTY);
};

void dbbconfig::AvgColorGradient_sliderMoved( int avggradient )
{
	updateAvgColorGradientText(avggradient);
	setState(dbbconfig::DIRTY);
};

void dbbconfig::SaveQuality_sliderMoved( int savequality )
{
	updateSaveQualityText(savequality);
	setState(dbbconfig::DIRTY);
};

void dbbconfig::SaveQuality_valueChanged( int savequality )
{
	updateSaveQualityText(savequality);
};

void dbbconfig::AvgLightness_valueChanged( int lightness )
{
	updateAvgLightnessText(lightness);
};

void dbbconfig::AvgColorGradient_valueChanged( int gradient )
{
	updateAvgColorGradientText(gradient);
};

void dbbconfig::updateSaveQualityText(int quality)
{
	QString s;
	s.sprintf("%d",quality );
	labelSaveQuality->setText(s);
};

void dbbconfig::updateAvgLightnessText(int lightness)
{
	QString s;
	s.sprintf("%d", lightness);
	labelAvgLightnessValue->setText(s);
};

void dbbconfig::updateAvgColorGradientText(int gradient)
{
	QString s;
	s.sprintf("%d", gradient);
	labelAvgColorGradientValue->setText(s);	
};

void dbbconfig::BtnCancel_clicked()
{
	btnReset_clicked();
	close();
};

void dbbconfig::BtnOk_clicked()
{
	saveConfiguration();
	close();	
};

void dbbconfig::btnHelpPath_clicked()
{
	QString sDir;
	Q3FileDialog* pDlg = new Q3FileDialog(this, 0, true);
	pDlg->setMode(Q3FileDialog::DirectoryOnly);
	pDlg->setWindowTitle("Set Help Path");
	if (pDlg->exec() == QDialog::Accepted)
	{
		sDir = pDlg->selectedFile();
		if (sDir.isEmpty() != true)
		{
			sHelpDir = sDir;
		}
	}
};

void dbbconfig::ckboxDisableThreshhold_clicked()
{
	if (ckboxDisableThreshhold->isChecked() == true)
	{
		enableThreshholdSliders(false);	
		bThreshholdsDisabled = true;
	}
	else
	{
		enableThreshholdSliders(true);	
		bThreshholdsDisabled = false;
	}
}

void dbbconfig::BtnOutputDir_clicked()
{
	QString sDir;
	Q3FileDialog* pDlg = new Q3FileDialog(this, 0, true);
	pDlg->setMode(Q3FileDialog::DirectoryOnly);
	pDlg->setWindowTitle("Set Output Path");
	if (pDlg->exec() == QDialog::Accepted)
	{
		sDir = pDlg->selectedFile();
		if (sDir.isEmpty() != true)
		{
			lineEditOutputDir->setText(sDir);
			labelRelOutputDir->setText("Output Path");
		}
	}
}

void dbbconfig::outDirChanged(const QString&)
{
	setState(dbbconfig::DIRTY);
}

void dbbconfig::ckboxDisableHotpixels_clicked()
{
	if (ckboxDisableHotpixels->checkState() == Qt::Checked)
	{
		bHotPixelsDisabled = true;
		ckboxHotPixelOnly->setEnabled(false);
	}
	else
	{
		bHotPixelsDisabled = false;
		ckboxHotPixelOnly->setEnabled(true);
	}
	
	setState(dbbconfig::DIRTY);
}

void dbbconfig::ckboxHotPixelOnly_clicked()
{
	if (ckboxHotPixelOnly->checkState() == Qt::Checked)
	{
		bHotPixelsOnly = true;
		ckboxDisableHotpixels->setEnabled(false);
		
		enableThreshholdSliders(false);	
		ckboxDisableThreshhold->setEnabled(false);
	}
	else
	{
		bHotPixelsOnly = false;
		ckboxDisableHotpixels->setEnabled(true);
		if (ckboxDisableThreshhold->checkState() == Qt::Unchecked)
		{
			enableThreshholdSliders(true);	
		}
		ckboxDisableThreshhold->setEnabled(true);
	}
	
	setState(dbbconfig::DIRTY);
}
		
void dbbconfig::btnReset_clicked()
{
	pMainWin->appSettings = savedSettings;
	initWidgets();	
	
	setState(dbbconfig::SAVED);
}

void dbbconfig::btnDefault_clicked()
{
	pMainWin->appSettings.saveImgQuality = defSaveImgQuality;
	pMainWin->appSettings.avgColorLightness = defAvgColorLightness;
	pMainWin->appSettings.avgColorRange = defAvgColorRange;
	pMainWin->appSettings.sHelpPath = defSHelpPath;
	pMainWin->appSettings.bThreshholdsDisabled = defBThreshholdsDisabled;
	pMainWin->appSettings.sLastDir = defSLastDir;
	pMainWin->appSettings.markerColor = defMarkerColor;
	pMainWin->appSettings.sOutputDir = defSOutputDir;
	pMainWin->appSettings.bHotpixelsDisabled = defBHotpixelsDisabled;
	pMainWin->appSettings.bHotpixelsOnly = defBHotpixelsOnly;
	initWidgets();	
	
	setState(dbbconfig::SAVED);
}

void dbbconfig::btnApply_clicked()
{
	saveConfiguration();
}

void dbbconfig::enableThreshholdSliders(bool bEnabled)
{
	sliderAvgLightness->setEnabled(bEnabled);
	sliderAvgColorGradient->setEnabled(bEnabled);
	labelAvgLightness->setEnabled(bEnabled);
	labelAvgLightnessValue->setEnabled(bEnabled);
	labelAvgColorGradient->setEnabled(bEnabled);	
	labelAvgColorGradientValue->setEnabled(bEnabled);
	
	setState(dbbconfig::DIRTY);
}

void dbbconfig::initWidgets()
{
	initSliders();
	
	bThreshholdsDisabled = pMainWin->appSettings.bThreshholdsDisabled;
	enableThreshholdSliders(!bThreshholdsDisabled);
	ckboxDisableThreshhold->setChecked(bThreshholdsDisabled);
	
	
	QDir outDir(pMainWin->appSettings.sOutputDir);
	
	if (outDir.isRelative()) labelRelOutputDir->setText("Relative Path");
	else labelRelOutputDir->setText("Output Path");
	
	lineEditOutputDir->setText(pMainWin->appSettings.sOutputDir);
	
	if (pMainWin->appSettings.bHotpixelsDisabled == true)
	{
		ckboxDisableHotpixels->setCheckState(Qt::Checked);
		ckboxHotPixelOnly->setEnabled(false);
	}
	else
	{
		ckboxDisableHotpixels->setCheckState(Qt::Unchecked);
		ckboxHotPixelOnly->setEnabled(true);
	}
	
	if (pMainWin->appSettings.bHotpixelsOnly == true)
	{
		ckboxHotPixelOnly->setCheckState(Qt::Checked);
		ckboxDisableHotpixels->setEnabled(false);
		ckboxDisableThreshhold->setEnabled(false);
		enableThreshholdSliders(false);
	}
	else
	{
		ckboxHotPixelOnly->setCheckState(Qt::Unchecked);
		ckboxDisableHotpixels->setEnabled(true);
		ckboxDisableThreshhold->setEnabled(true);
		enableThreshholdSliders(!pMainWin->appSettings.bThreshholdsDisabled);
	}
	
	sHelpDir = pMainWin->appSettings.sHelpPath;
	
}

void dbbconfig::initSliders()
{
	int SaveQuality = pMainWin->appSettings.saveImgQuality;
	int AvgLightness = pMainWin->appSettings.avgColorLightness;
	int AvgColorRange = pMainWin->appSettings.avgColorRange;
	
	int saveQualityMin = 30;
	int saveQualityMax = 100;
	
	int avgLightnessMin = 0;
	int avgLightnessMax = 255;
	
	int avgColorRangeMin = 0;
	int avgColorRangeMax = 255;
	
	sliderSaveQuality->setFocusPolicy(Qt::StrongFocus);
	sliderAvgLightness->setFocusPolicy(Qt::StrongFocus);
	sliderAvgColorGradient->setFocusPolicy(Qt::StrongFocus);
			
	sliderSaveQuality->setMinValue(saveQualityMin);
	sliderSaveQuality->setMaxValue(saveQualityMax);
	sliderSaveQuality->setValue(SaveQuality);
	SaveQuality_sliderMoved(SaveQuality);
	
	sliderAvgLightness->setMinValue(avgLightnessMin);
	sliderAvgLightness->setMaxValue(avgLightnessMax);
	sliderAvgLightness->setValue(AvgLightness);
	AvgLightness_sliderMoved(AvgLightness);
	
	sliderAvgColorGradient->setMinValue(avgColorRangeMin);
	sliderAvgColorGradient->setMaxValue(avgColorRangeMax);
	sliderAvgColorGradient->setValue(AvgColorRange);
	AvgColorGradient_sliderMoved(AvgColorRange);
};

void dbbconfig::saveConfiguration()
{
	pMainWin->appSettings.saveImgQuality = sliderSaveQuality->value();
	pMainWin->appSettings.avgColorLightness = sliderAvgLightness->value();
	pMainWin->appSettings.avgColorRange = sliderAvgColorGradient->value();
	pMainWin->appSettings.sHelpPath = sHelpDir;
	pMainWin->appSettings.sOutputDir = lineEditOutputDir->text();
	
	if (ckboxDisableHotpixels->checkState() == Qt::Checked) pMainWin->appSettings.bHotpixelsDisabled = true;
	else pMainWin->appSettings.bHotpixelsDisabled = false;
	
	if (ckboxHotPixelOnly->checkState() == Qt::Checked) pMainWin->appSettings.bHotpixelsOnly = true;
	else pMainWin->appSettings.bHotpixelsOnly = false;
	
	if (ckboxDisableThreshhold->checkState() == Qt::Checked) pMainWin->appSettings.bThreshholdsDisabled = true;
	else pMainWin->appSettings.bThreshholdsDisabled = false;

	QString s = this->parent()->objectName();
	if (s == "dbbatch_base")
	{
		((dbbatch*)(this->parent()))->hotPixelProcessingChanged(pMainWin->appSettings.bHotpixelsOnly);
	}
	
	setState(dbbconfig::SAVED);
};

void dbbconfig::setState(int st)
{
	switch (st)
	{
		case dbbconfig::NEW:
			btnApply->setEnabled(false);
			btnReset->setEnabled(false);
			break;
		case dbbconfig::DIRTY:
			btnApply->setEnabled(true);
			btnReset->setEnabled(true);
			break;
		case dbbconfig::SAVED:
			btnApply->setEnabled(false);
			btnReset->setEnabled(true);
			break;
		default:
			break;
	}
	
	state = st;
}












