//
// C++ Implementation: exif
//
// Description: 
//
//
// Author: ,,, <heinzg@denizen>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include <iostream>

#include <qstring.h>

#include <image.hpp>
#include <exif.hpp>

#include "exif.h"
#include "errors.h"


int copyExifData(const char *infile, const char *outfile)
{
	int error = ERR_NOERR;
	
	try {
		//Open the input file
		Exiv2::Image::AutoPtr inputimage = Exiv2::ImageFactory::open(infile);
		if (inputimage.get() == 0)
		{
			ERROR(ERR_NULL_OBJECT, "copyExifData, inputimage is 0");
			error = ERR_NULL_OBJECT;
		}
		
		//Get the exif data from the input file
		inputimage->readMetadata();
		Exiv2::ExifData &exifData = inputimage->exifData();
		if (exifData.empty()) 
		{
			ERROR(ERR_NULL_OBJECT, "copyExifData, exifData is 0");
			error = ERR_NULL_OBJECT;
		}
		
		//Open the output file
		Exiv2::Image::AutoPtr outimage = Exiv2::ImageFactory::open(outfile);
		if (outimage.get() == 0)
		{
			ERROR(ERR_NULL_OBJECT, "copyExifData, outimage is 0");
			error = ERR_NULL_OBJECT;
		}
		
		//Write the exif data extracted from the input file
		//to the output file
		outimage->setExifData(exifData);
		outimage->writeMetadata();
	}
	
	catch (Exiv2::AnyError& e)
	{
		ERROR(ERR_EXIV2_ERROR, "Error from Exiv2");
		error = ERR_EXIV2_ERROR;	
	}
	
	return error;
}


