//mainwin.cpp
#include <stdlib.h>

#include <q3popupmenu.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <q3canvas.h>
#include <qpixmap.h>
#include <q3filedialog.h>
#include <qstring.h>
#include <qpainter.h>
#include <qimage.h>
#include <qapplication.h>
#include <qrect.h>
#include <qsettings.h>
#include <qdir.h>
#include <q3textbrowser.h>
#include <qmime.h>
//Added by qt3to4:
#include <QCloseEvent>
#include <QLabel>

#include "mainwin.h" 
#include "canvasview.h"
#include "canvas.h"
#include "bunny.h"
#include "blaster.h"
#include "settings.h"
#include "dbbconfig.h"
#include "dbbatch.h"
#include "version.h"
#include "helpviewer.h"
#include "hotpixel.h"
#include "errors.h"
#include "exif.h"

using namespace Qt;

//#define TESTBLAST
//#define DEBUGOUT

#ifdef DEBUGOUT
#include <iostream>
#define cout std::cout
#define endl std::endl
#endif //DEBUGOUT

extern bunnies * pBunnies;

mainwin * pMainWin;

mainwin::mainwin() : Q3MainWindow()
{
	ppixmap = 0;
	
	createMenu();
	createStatusBar();
	initCanvas();
	pMainWin = this;
	pstatusbar->message("Idle");
	
	loadSettings();
	
	ppixlist = new pixlist();

	setState(IDLE, 0);
};


mainwin::~mainwin()
{

};

void mainwin::createMenu()
{
	//File menu *******************
	pfileMenu = menuBar()->addMenu("&File");
	
	pcreateRefAct = new QAction("Create &reference file", this);
	pcreateRefAct->setShortcut(CTRL+Key_R);
	connect(pcreateRefAct, SIGNAL(triggered()), this, SLOT(createReferenceFile()));
	pfileMenu->addAction(pcreateRefAct);
	
	popenRefAct = new QAction("&Open reference file", this);
	popenRefAct->setShortcut(CTRL+Key_O);
	connect(popenRefAct, SIGNAL(triggered()), this, SLOT(openReferenceFile()));
	pfileMenu->addAction(popenRefAct);

	psaveRefAct = new QAction("&Save reference file", this);
	psaveRefAct->setShortcut(CTRL+Key_S);
	connect(psaveRefAct, SIGNAL(triggered()), this, SLOT(saveReferenceFile()));
	pfileMenu->addAction(psaveRefAct);
	
	pcloseRefAct = new QAction("&Close reference file", this);
	pcloseRefAct->setShortcut(CTRL+Key_C);
	connect(pcloseRefAct, SIGNAL(triggered()), this, SLOT(closeReferenceFile()));
	pfileMenu->addAction(pcloseRefAct);

#ifdef TESTBLAST	
	pfileMenu->addSeparator();
	ptestBlastAct = new QAction("&Test", this);
	ptestBlastAct->setShortcut(CTRL+Key_T);
	connect(ptestBlastAct, SIGNAL(triggered()), this, SLOT(testBlast()));
	pfileMenu->addAction(ptestBlastAct);
#endif	
	pfileMenu->addSeparator();
	
	popenImageAct = new QAction("O&pen image file", this);
	popenImageAct->setShortcut(CTRL+Key_P);
	connect(popenImageAct, SIGNAL(triggered()), this, SLOT(openImageFile()));
	pfileMenu->addAction(popenImageAct);
	
	psaveImageAct = new QAction("Sa&ve image file", this);
	psaveImageAct->setShortcut(CTRL+Key_V);
	connect(psaveImageAct, SIGNAL(triggered()), this, SLOT(saveImageFile()));
	pfileMenu->addAction(psaveImageAct);
	
	psaveImageAsAct = new QAction("Sav&e image file as", this);
	psaveImageAsAct->setShortcut(CTRL+Key_E);
	connect(psaveImageAsAct, SIGNAL(triggered()), this, SLOT(saveImageFileAs()));
	pfileMenu->addAction(psaveImageAsAct);
	
	pcloseImageAct = new QAction("C&lose image file", this);
	pcloseImageAct->setShortcut(CTRL+Key_L);
	connect(pcloseImageAct, SIGNAL(triggered()), this, SLOT(closeImageFile()));
	pfileMenu->addAction(pcloseImageAct);
	
	pfileMenu->addSeparator();
	
	pexitAct = new QAction("E&xit", this);
	pexitAct->setShortcut(CTRL+Key_X);
	connect(pexitAct, SIGNAL(triggered()), this, SLOT(close()));
	pfileMenu->addAction(pexitAct);
	
	//Settings menu *******************
	psettingsMenu = menuBar()->addMenu("&Settings");
	
	pconfigureAct = new QAction("Conf&igure dbblast...", this);
	pconfigureAct->setShortcut(CTRL+Key_I);
	connect(pconfigureAct, SIGNAL(triggered()), this, SLOT(configureDbblast()));
	psettingsMenu->addAction(pconfigureAct);
	
	//Tools menu
#ifdef TESTBLAST	
	ptoolsMenu = menuBar()->addMenu("Too&ls");
#else
	ptoolsMenu = menuBar()->addMenu("&Tools");
#endif
	pbatchBlastAct = new QAction("&Batch blast...", this);
	pbatchBlastAct->setShortcut(CTRL+Key_B);
	connect(pbatchBlastAct, SIGNAL(triggered()), this, SLOT(batchBlast()));
	ptoolsMenu->addAction(pbatchBlastAct);
	
	//Help menu *******************
	phelpMenu = menuBar()->addMenu("&Help");
	
	phelpAct = new QAction("&Help...", this);
	phelpAct->setShortcut(CTRL+Key_H);
	connect(phelpAct, SIGNAL(triggered()), this, SLOT(showHelp()));
	phelpMenu->addAction(phelpAct);
	
	phelpMenu->addSeparator();
			
	paboutAct = new QAction("&About dbblast...", this);
	paboutAct->setShortcut(CTRL+Key_A);
	connect(paboutAct, SIGNAL(triggered()), this, SLOT(aboutDbblast()));
	phelpMenu->addAction(paboutAct);
			
	paboutQtAct = new QAction("Abo&ut Qt...", this);
	paboutQtAct->setShortcut(CTRL+Key_U);
	connect(paboutQtAct, SIGNAL(triggered()), this, SLOT(aboutQt()));
	phelpMenu->addAction(paboutQtAct);
	
}

void mainwin::createStatusBar()
{
	pstatusbar = new QStatusBar(this);
	pstatuslabel = new QLabel(this);
	pstatuslabel->setMinimumWidth(80);
	pstatusbar->addWidget(pstatuslabel, 0, true);
}

void mainwin::initCanvas()  
{
	pcanvasview = new canvasview(this);
	pcanvas = new canvas(640,480);
	pcanvasview->setCanvas(pcanvas);  
	setCentralWidget(pcanvasview);
	pcanvasview->show();
}

void mainwin::setState(int st, int substate)
{
	state = st;
	
	switch (st)
	{
		case IDLE:
			pcreateRefAct->setEnabled(true);
			pbatchBlastAct->setEnabled(true);
			psaveRefAct->setEnabled(false);
			popenRefAct->setEnabled(false);
			pcloseRefAct->setEnabled(false);
			popenImageAct->setEnabled(true);
			psaveImageAct->setEnabled(false);
			psaveImageAsAct->setEnabled(false);
			pcloseImageAct->setEnabled(false);
			break;
		case CREATE_REFERENCE:
			popenImageAct->setEnabled(false);
			pbatchBlastAct->setEnabled(false);
			popenRefAct->setEnabled(true);
			psaveRefAct->setEnabled(false);
			pcloseRefAct->setEnabled(true);
			break;
		case OPEN_IMG:
			pcreateRefAct->setEnabled(false);
			pbatchBlastAct->setEnabled(false);
			psaveImageAct->setEnabled(false);
			psaveImageAsAct->setEnabled(true);
			pcloseImageAct->setEnabled(true);
			break;
		case BATCH_BLAST:
			break;
		default:
			break;	
	} //end switch st
	
	switch (substate)
	{
		case OPEN_IMG_DIRTY:
			psaveImageAct->setEnabled(true);
			break;
		case OPEN_IMG_SAVED:
			psaveImageAct->setEnabled(false);
			break;
		case CREATE_REFERENCE_DIRTY:
			psaveRefAct->setEnabled(true);
			break;
		case CREATE_REFERENCE_SAVED:
			psaveRefAct->setEnabled(false);
			break;
		default:
			break;
	} //end switch substate
}

void mainwin::getFileOpenName(QString & s)
{
	QString sDir;
	if (appSettings.sLastDir == "")	sDir = QDir::home().absPath();
	else sDir = appSettings.sLastDir;
	
	s = Q3FileDialog::getOpenFileName(
					sDir,
					"Images (*.jpg;*.jpeg;*.png);;All files (*.*)",
					this,
					"open file dialog",
					"Choose a file" );
	
}

void mainwin::showImage(QString & s)
{
	if (s.isEmpty()) return;
	
	QString sCaption("dbblast");
	setCaption(sCaption + " - " + s);
	
	//Create a QPixmap, then load into the canvas
	if (ppixmap != 0) delete ppixmap;
	
	ppixmap = new QPixmap(s);
	pcanvas->resize(ppixmap->width(), ppixmap->height());
	pcanvas->setBackgroundPixmap(*ppixmap);
	pcanvas->update();
} 



/////////////////////////////////////////////////////////////////////////////////////////////////////////
// Slots
/////////////////////////////////////////////////////////////////////////////////////////////////////////


void mainwin::createReferenceFile()
{
	if (psaveRefAct->isEnabled() == true)
	{
		int ret = QMessageBox::warning(this, "dbblast", "Save changes?", QMessageBox::Save | QMessageBox::Cancel | QMessageBox::No );
		if (ret == QMessageBox::Save)
		{
			if (saveReferenceFile() != ERR_NOERR) return;
		}
		else if (ret == QMessageBox::Cancel)
		{
			return;
		}
	}
	
	getFileOpenName(sfilename);
	
	if (sfilename.isEmpty()) return;
	
	QFileInfo f(sfilename);
	appSettings.sLastDir = f.dirPath(true);
	
	showImage(sfilename);
	
	if (pBunnies != 0) delete pBunnies;
	pBunnies = new bunnies();
	
	pstatusbar->message("Right click to show menu.");
	pstatuslabel->setText("");
	setState(CREATE_REFERENCE, 0);
}

void mainwin::closeReferenceFile()
{
	if (psaveRefAct->isEnabled() == true)
	{
		int ret = QMessageBox::warning(this, "dbblast", "Save changes?", QMessageBox::Save | QMessageBox::Cancel | QMessageBox::No );
		if (ret == QMessageBox::Save)
		{
			if (saveReferenceFile() != ERR_NOERR) return;
		}
		else if (ret == QMessageBox::Cancel)
		{
			return;
		}
	}
	
	cleanupBunniesAndPixmap();
	setState(IDLE, 0);
}

void mainwin::cleanupBunniesAndPixmap()
{
	if (pBunnies != 0) 
	{
		delete pBunnies;
		pBunnies = 0;
	}
	if (ppixmap != 0) 
	{
		delete ppixmap;
		ppixmap = 0;
	}
	pcanvas->resize(0, 0);
	pcanvas->update();
	
	pstatusbar->message("Idle");
	pstatuslabel->setText("");
	
	setCaption("dbblast");
}

void mainwin::testBlast()
{
	/*
	if (pBunnies == 0) return;
	if (ppixmap == 0) return;
	
	blaster *pblast = new blaster();
	
	pblast->interpolate(pBunnies, 0, ppixmap);
	
	pcanvas->setAllChanged();
	pcanvas->update();
	
	delete pblast;
	pblast = 0;
	*/
	
	/*
	QStringList list;
	if (hotpixel::getHotPixelList(list) == ERR_NOFILE) cout << "no file!!" << endl;
	
	hotpixel hp(list);
	
	QImage img("dsc_4423e.jpg");
	hp.processHotPixels(img);
	img.save("out.jpg", 0, 96);
	*/
}


int mainwin::saveReferenceFile()
{
	QString sDir;
	if (appSettings.sLastDir == "")	sDir = QDir::home().absPath();
	else sDir = appSettings.sLastDir;
	
	QString s = Q3FileDialog::getSaveFileName(
		sDir,
		"Reference files (*.dbr);;All files (*.*)",
		this,
		"save file dialog",
		"Choose a filename to save under" );

	if (pBunnies == 0) return ERR_NULL_OBJECT;

	if ( !s.isEmpty() )
	{
		QFileInfo fi(s);
		if ( !fi.suffix().contains("dbr", false) )
		{
			s += ".dbr";
		}

		//put up nag message if file exists
		QFile fo(s);
		if ( fo.exists() )
		{
			QString sMsg = "Overwrite " + s + "?";
			int ret = QMessageBox::warning(this, "dbblast", sMsg, QMessageBox::Yes | QMessageBox::No );
			if ( ret == QMessageBox::No )
			{
				return ERR_NOFILE;
			}
		}

		QFileInfo f(s);
		pBunnies->save( (char*) s.latin1() );
		appSettings.sLastDir = f.dirPath(true);
		setState(CREATE_REFERENCE,CREATE_REFERENCE_SAVED);
		return ERR_NOERR;
	}
	
	return ERR_NOFILE;
}	

void mainwin::openReferenceFile()
{
	if (pBunnies == 0) return;
	if (ppixmap == 0) return;
	
	if (psaveRefAct->isEnabled() == true)
	{
		int ret = QMessageBox::warning(this, "dbblast", "Save changes?", QMessageBox::Save | QMessageBox::Cancel | QMessageBox::No );
		if (ret == QMessageBox::Save)
		{
			if (saveReferenceFile() != ERR_NOERR) return;
		}
		else if (ret == QMessageBox::Cancel)
		{
			return;
		}
	}
	
	QString sDir;
	if (appSettings.sLastDir == "")	sDir = QDir::home().absPath();
	else sDir = appSettings.sLastDir;
	
	QString s = Q3FileDialog::getOpenFileName(
		sDir,
		"Reference files (*.dbr);;All files (*.*)",
		this,
		"open file dialog",
		"Choose a file" );

	if ( !s.isEmpty() )
	{
		pBunnies->loadFromFile( (char*) s.latin1() );
		pcanvas->setAllChanged();
		pcanvas->update();
		QFileInfo f(s);
		appSettings.sLastDir = f.dirPath(true);
		setState(CREATE_REFERENCE, 0);
	}
}

void mainwin::configureDbblast()
{
	dbbconfig * pconfig = new dbbconfig(this);
	pconfig->show();
}


void mainwin::showHelp()
{
	helpviewer* phelp = new helpviewer();
	Q3MimeSourceFactory* pMime =  phelp->textBrowser->mimeSourceFactory();
	QStringList sList;
	sList += appSettings.sHelpPath;
	pMime->setFilePath(sList);
	phelp->textBrowser->setSource("index.html");
	phelp->show();
}

void mainwin::aboutDbblast()
{
	QString s;
	s.sprintf("dbblast: Version %s\n(c) 2007 Heinz Graupe\nDistributed under terms of GPL v.2", DBBLASTVERSION);
	QMessageBox::about(this, "About dbblast", s);
}

void mainwin::aboutQt()
{
	QMessageBox::aboutQt(this);
}

void mainwin::openImageFile()
{
	if (psaveImageAct->isEnabled() == true)
	{
		int ret = QMessageBox::warning(this, "dbblast", "Save changes?", QMessageBox::Save | QMessageBox::Cancel | QMessageBox::No );
		if (ret == QMessageBox::Save)
		{
			if (saveImageFileAs() != ERR_NOERR) return;
		}
		else if (ret == QMessageBox::Cancel)
		{
			return;
		}
	}
	
	getFileOpenName(sfilename);
	
	if (sfilename.isEmpty()) return;
	
	QFileInfo f(sfilename);
	appSettings.sLastDir = f.dirPath(true);
	
	showImage(sfilename);
	
	if (pBunnies != 0) delete pBunnies;
	pBunnies = new bunnies();
	
	pstatusbar->message("Right click to show menu.");
	pstatuslabel->setText("");
	pixList()->clear();
	setState(OPEN_IMG, 0);
}

int mainwin::saveImageFile()
{
	//put up overwrite nag message
	QString sMsg = "Overwrite " + sfilename + "?";
	int ret = QMessageBox::warning(this, "dbblast", sMsg, QMessageBox::Yes | QMessageBox::No );
	if ( ret == QMessageBox::No )
	{
		return ERR_NOFILE;
	}
	
	saveImageAndCopyExifData(sfilename, sfilename);
	pixList()->clear();
	setState(OPEN_IMG, OPEN_IMG_SAVED);
	return ERR_NOERR;
}

int mainwin::saveImageFileAs()
{
	//get the output filename
	QString sDir;
	if (appSettings.sLastDir == "")	sDir = QDir::home().absPath();
	else sDir = appSettings.sLastDir;
	
	QString s = Q3FileDialog::getSaveFileName(
			sDir,
	"Images (*.jpg;*.jpeg;*.png)",
	this,
	"save file dialog",
	"Choose a filename to save under" );
	
	if ( s.isEmpty() ) return ERR_NOFILE;

	//put up nag message if file exists
	QFile f(s);
	if ( f.exists() )
	{
		QString sMsg = "Overwrite " + s + "?";
		int ret = QMessageBox::warning(this, "dbblast", sMsg, QMessageBox::Yes | QMessageBox::No );
		if ( ret == QMessageBox::No )
		{
			return ERR_NOFILE;
		}
	}
	
	QFileInfo info(s);
	if (info.suffix().contains("jpg", false) == false && 
		info.suffix().contains("jpeg", false) == false &&
		info.suffix().contains("png", false) == false )
	{
		QMessageBox::critical(this, "dbblast", "Invalid filename");
		return ERR_NOFILE;
	}
	
	saveImageAndCopyExifData(sfilename, s);
	pixList()->clear();
	//reset popup memu items
	setState(OPEN_IMG, OPEN_IMG_SAVED);
	
	return ERR_NOERR;
}

int mainwin::saveImageAndCopyExifData(QString& sIn, QString&sOut)
{
	if (ppixmap == 0) return ERR_NULL_OBJECT;
	
	//convert pixmap to QImage and save
	QImage img = ppixmap->toImage();
	if ( !img.isNull() )
	{
		QFileInfo info(sOut);
		QString sext = info.suffix();
		QString stmp = "tmpout." + sext;
		
		img.save( stmp, 0, appSettings.saveImgQuality );
		copyExifData(sIn.latin1() , stmp.latin1() );
		
		QFile f(stmp);
		if ( f.exists())
		{
			if (QFile::exists(sOut))
			{
				f.remove(sOut);
			}
			f.rename(sOut);
		}
	}
	
	return ERR_NOERR;
}

void mainwin::closeImageFile()
{
	if (psaveImageAct->isEnabled() == true)
	{
		int ret = QMessageBox::warning(this, "dbblast", "Save changes?", QMessageBox::Save | QMessageBox::Cancel | QMessageBox::No );
		if (ret == QMessageBox::Save)
		{
			if (saveImageFileAs() != ERR_NOERR) return;
		}
		else if (ret == QMessageBox::Cancel)
		{
			return;
		}
	}
	pixList()->clear();
	setState(IDLE, 0);	
	cleanupBunniesAndPixmap();	
}


//////////////////////////////////////////////////////////////////////
//End slots

void mainwin::closeEvent(QCloseEvent *e)
{
	if ( psaveRefAct->isEnabled() == true )
	{
		int ret = QMessageBox::warning(this, "dbblast", "Save changes?", QMessageBox::Save | QMessageBox::Cancel | QMessageBox::No );
		if (ret == QMessageBox::Save)
		{
			if (saveReferenceFile() != ERR_NOERR)
			{ 
				e->ignore();
				return;
			}
		}
		else if (ret == QMessageBox::Cancel)
		{
			e->ignore();
			return;
		}
	}
	
	if ( psaveImageAct->isEnabled() == true )
	{
		int ret = QMessageBox::warning(this, "dbblast", "Save changes?", QMessageBox::Save | QMessageBox::Cancel | QMessageBox::No );
		if (ret == QMessageBox::Save)
		{
			if (saveImageFileAs() != ERR_NOERR) 
			{
				e->ignore();
				return;
			}
		}
		else if (ret == QMessageBox::Cancel)
		{
			e->ignore();
			return;
		}
	}
	
	saveSettings();	
	e->accept();
}

//appSettings is defined in settings.h
//settings are loaded from and saved to ~/.qt/dbblastrc
//the call to setPath() only has an effect on Win32.
void mainwin::loadSettings()
{
	QSettings set("VCS", "dbblast");
	
	appSettings.saveImgQuality = set.readNumEntry("/dbblast/saveQuality", defSaveImgQuality);
	appSettings.avgColorLightness = set.readNumEntry("/dbblast/avgColorLightness", defAvgColorLightness);
	appSettings.avgColorRange = set.readNumEntry("/dbblast/avgColorRange", defAvgColorRange);
	appSettings.sHelpPath = set.readEntry("/dbblast/sHelpPath", defSHelpPath);
	appSettings.bThreshholdsDisabled = set.readBoolEntry("/dbblast/bThreshholdsDisabled", defBThreshholdsDisabled);
	appSettings.sLastDir = set.readEntry("/dbblast/sLastDir", defSLastDir);
	appSettings.markerColor = set.readNumEntry("/dbblast/markerColor", defMarkerColor);
	appSettings.sOutputDir = set.readEntry("/dbblast/sOutputDir", defSOutputDir);
	appSettings.bHotpixelsDisabled = set.readBoolEntry("/dbblast/bHotpixelsDisabled", defBHotpixelsDisabled);
	appSettings.bHotpixelsOnly = set.readBoolEntry("/dbblast/bHotpixelsOnly", defBHotpixelsOnly);
}

void mainwin::saveSettings()
{
	QSettings set("VCS", "dbblast");

	set.writeEntry("/dbblast/saveQuality", appSettings.saveImgQuality);
	set.writeEntry("/dbblast/avgColorLightness", appSettings.avgColorLightness);
	set.writeEntry("/dbblast/avgColorRange", appSettings.avgColorRange);
	set.writeEntry("/dbblast/sHelpPath", appSettings.sHelpPath);
	set.writeEntry("/dbblast/bThreshholdsDisabled", appSettings.bThreshholdsDisabled);
	set.writeEntry("/dbblast/sLastDir", appSettings.sLastDir);
	set.writeEntry("/dbblast/markerColor", appSettings.markerColor);
	set.writeEntry("/dbblast/sOutputDir", appSettings.sOutputDir);
	set.writeEntry("/dbblast/bHotpixelsDisabled", appSettings.bHotpixelsDisabled);
	set.writeEntry("/dbblast/bHotpixelsOnly", appSettings.bHotpixelsOnly);
}

void mainwin::batchBlast()
{
	dbbatch * pbatch = new dbbatch(this);
	pbatch->show();

};



